"""This file is the initialization module for the `sosia.classes`. It imports the `Scientist` and
`Original` classes from the respective modules.
"""

from sosia.classes.scientist import *
from sosia.classes.original import *
